from django.shortcuts import render, redirect
from django.http import JsonResponse
from django.conf import settings
import time
import os
import rust_crud_api # Custom Rust Pyo3 Library 🦀

db_url = settings.DATABASE_URL

def register_view(request):
    """
    Handles user registration. Expects POST with:
    'name', 'email', 'password', 'username', 'studentid', 'startyear', 'endyear',
    and optionally 'profilepicture'.
    """
    db_url = settings.DATABASE_URL
    context = {"form_data": {}}  # Store form data and errors

    if request.method == 'POST':
        # Retrieve user input
        form_data = {
            "name": request.POST.get("name", "").strip(),
            "email": request.POST.get("email", "").strip(),
            "password": request.POST.get("password", "").strip(),
            "username": request.POST.get("username", "").strip(),
            "studentid": request.POST.get("studentid", "").strip(),
            "startyear": request.POST.get("startyear", "").strip(),
            "endyear": request.POST.get("endyear", "").strip(),
        }

        # Preserve form data for repopulation
        context["form_data"] = form_data

        # 1. Check Required Fields
        if not all(form_data.values()):
            context["error"] = "All fields are required."
            return render(request, "myapp/register.html", context)

        # 2. Convert Numeric Fields with Error Handling
        try:
            form_data["studentid"] = int(form_data["studentid"])
            form_data["startyear"] = int(form_data["startyear"])
            form_data["endyear"] = int(form_data["endyear"])
        except ValueError:
            context["error"] = "Student ID, Start Year, and End Year must be numbers."
            return render(request, "myapp/register.html", context)

        # 3. Handle Profile Picture Upload (if provided)
        profilepicture_file = request.FILES.get("profilepicture")
        saved_path = None

        if profilepicture_file:
            try:
                profile_dir = os.path.join(settings.MEDIA_ROOT, "profile_pics")
                os.makedirs(profile_dir, exist_ok=True)  # Ensure directory exists

                filename = profilepicture_file.name
                save_path = os.path.join(profile_dir, filename)

                with open(save_path, "wb+") as destination:
                    for chunk in profilepicture_file.chunks():
                        destination.write(chunk)

                # Store relative media path in DB
                saved_path = f"/media/profile_pics/{filename}"
            except Exception as e:
                context["error"] = f"Error saving profile picture: {str(e)}"
                return render(request, "myapp/register.html", context)

        # Debugging Information
        print("DEBUG: profilepicture_file is:", profilepicture_file)
        print("DEBUG: saved_path is:", saved_path)

        # 4. Call Rust API to Create User
        try:
            rust_crud_api.create_user(
                db_url,
                form_data["name"],
                form_data["email"],
                form_data["password"],
                form_data["username"],
                form_data["studentid"],
                form_data["startyear"],
                form_data["endyear"],
                saved_path,
                True,  # First-time login flag
            )
            return redirect("login")  # Redirect to login page after success
        except Exception as e:
            error_message = str(e)

            # Handle PostgreSQL unique constraint violation (Student ID exists)
            if "Student ID already exists" in error_message:
                context["error"] = "A user with this Student ID already exists. Please choose a different ID."
            else:
                context["error"] = f"An error occurred: {error_message}"

            print("DEBUG: Rust API Error:", error_message)  # Log error for debugging
            return render(request, "myapp/register.html", context)

    # Render registration form for GET requests
    return render(request, "myapp/register.html", context)

def account_view(request):
    """
    Display the account page for the logged-in user.
    """
    user_id = request.session.get('user_id')
    if not user_id:
        return redirect('login')
    db_url = settings.DATABASE_URL
    try:
        user = rust_crud_api.get_user(db_url, int(user_id))
        if user is None:
            return JsonResponse({'error': 'User not found.'}, status=404)
        return render(request, 'myapp/account.html', {'user': user})
    except Exception as e:
        return JsonResponse({'error': str(e)}, status=500)


def delete_account_view(request):
    """
    Delete the account of the logged-in user.
    """
    user_id = request.session.get('user_id')
    if not user_id:
        return redirect('login')
    db_url = settings.DATABASE_URL
    if request.method == 'POST':
        try:
            success = rust_crud_api.delete_user(db_url, int(user_id))
            if success:
                request.session.flush()  # Clear the session after deletion.
                return redirect('register')
            else:
                return JsonResponse({'error': 'Account deletion failed.'}, status=500)
        except Exception as e:
            return JsonResponse({'error': str(e)}, status=500)
    return render(request, 'myapp/delete_account.html')

def welcome_view(request):
    return render(request, 'myapp/welcome_view.html')

def edit_account_view(request):
    db_url = settings.DATABASE_URL
    context = {}
    user_id = request.session.get('user_id')
    
    # If user is not logged in, redirect them to login
    if not user_id:
        return redirect('login')
    
    if request.method == 'POST':
        # Get form values
        new_name = request.POST.get('name', '').strip()
        new_email = request.POST.get('email', '').strip()
        new_username = request.POST.get('username', '').strip()
        new_studentid_str = request.POST.get('studentid', '').strip()
        new_startyear_str = request.POST.get('startyear', '').strip()
        new_endyear_str = request.POST.get('endyear', '').strip()
        
        # Handle profile picture upload, if any.
        profilepicture_file = request.FILES.get('profilepicture')
        saved_path = None
        if profilepicture_file:
            filename = profilepicture_file.name
            # Decide the folder inside MEDIA_ROOT for profile pics.
            profile_dir = os.path.join(settings.MEDIA_ROOT, 'profile_pics')
            os.makedirs(profile_dir, exist_ok=True)
            # Full filesystem path where file will be saved.
            save_path = os.path.join(profile_dir, filename)
            with open(save_path, 'wb+') as destination:
                for chunk in profilepicture_file.chunks():
                    destination.write(chunk)
            # Build the URL path to store in DB.
            saved_path = f"/media/profile_pics/{filename}"
        
        # Convert numeric fields; if conversion fails, re-render with an error.
        try:
            new_studentid = int(new_studentid_str) if new_studentid_str else None
        except ValueError:
            context['error'] = "Student ID must be a number."
            return render(request, 'myapp/edit_account.html', context)
        
        try:
            new_startyear = int(new_startyear_str) if new_startyear_str else None
        except ValueError:
            context['error'] = "Start year must be a number."
            return render(request, 'myapp/edit_account.html', context)
        
        try:
            new_endyear = int(new_endyear_str) if new_endyear_str else None
        except ValueError:
            context['error'] = "End year must be a number."
            return render(request, 'myapp/edit_account.html', context)
        
        # Call the flexible update_user function from your Rust API.
        # Here we pass None for fields the user hasn't changed.
        try:
            rust_crud_api.update_user(
                db_url,
                int(user_id),
                new_name if new_name else None,
                new_email if new_email else None,
                None,  # We're not updating the password here.
                new_username if new_username else None,
                new_studentid,
                new_startyear,
                new_endyear,
                saved_path,  # We leave firstlogin unchanged on account edits.
            )
            # Redirect to the account page after a successful update.
            return redirect('account')
        except Exception as e:
            context['error'] = f"An error occurred: {str(e)}"
            # Fall through to re-render the form with error message.
    
    # For GET (or if POST fails), fetch the current user details.
    try:
        user = rust_crud_api.get_user(db_url, int(user_id))
        if not user:
            return redirect('login')
    except Exception as e:
        context['error'] = f"Error fetching user data: {str(e)}"
        user = None
    
    context['user'] = user
    return render(request, 'myapp/edit_account.html', context)
