use crate::models::Event;
use pyo3::prelude::*;
use pyo3::exceptions::PyRuntimeError;
use postgres::{Client, NoTls};
use chrono::{NaiveDate, NaiveTime, NaiveDateTime};

fn pg_err(e: postgres::Error) -> PyErr {
    PyRuntimeError::new_err(e.to_string())
}

/// Create a new event
#[pyfunction]
pub fn create_event(
    db_url: &str,
    title: &str,
    description: Option<&str>,
    location: &str,
    date: &str,         // expected format: "YYYY-MM-DD"
    time: &str,         // expected format: "HH:MM:SS"
    created_by: i32,
    group_id: Option<i32>
) -> PyResult<()> {
    let mut client = Client::connect(db_url, NoTls).map_err(pg_err)?;

    let date_parsed = NaiveDate::parse_from_str(date, "%Y-%m-%d")
        .map_err(|e| PyRuntimeError::new_err(format!("Invalid date: {}", e)))?;
    let time_parsed = NaiveTime::parse_from_str(time, "%H:%M:%S")
        .map_err(|e| PyRuntimeError::new_err(format!("Invalid time: {}", e)))?;

    client.execute(
        "
        INSERT INTO events (title, description, location, date, time, created_by, group_id)
        VALUES ($1, $2, $3, $4, $5, $6, $7)
        ",
        &[&title, &description, &location, &date_parsed, &time_parsed, &created_by, &group_id]
    ).map_err(pg_err)?;

    Ok(())
}

#[pyfunction]
pub fn get_all_events(db_url: &str) -> PyResult<Vec<PyEvent>> {
    let mut client = Client::connect(db_url, NoTls).map_err(pg_err)?;

    let mut events = Vec::new();
    for row in client.query("SELECT * FROM events ORDER BY date, time", &[]).map_err(pg_err)? {
        let event = PyEvent {
            id: row.get("id"),
            title: row.get("title"),
            description: row.get("description"),
            location: row.get("location"),
            date: row.get::<_, chrono::NaiveDate>("date").to_string(),
            time: row.get::<_, chrono::NaiveTime>("time").to_string(),
            created_by: row.get("created_by"),
            group_id: row.get("group_id"),
            created_at: row.get::<_, chrono::NaiveDateTime>("created_at").to_string(),
        };
        events.push(event);
    }

    Ok(events)
}